#!/bin/csh -f

## Set Debug Parameter 
# DEBUG=1; erase all temporary files 
#       0; leave all temporary files 
set DEBUG=0
 
## Parameters for Photometry
# Radius within which detect object to do photometry
set PHOTRADIUS=200
# Radius of annulus to measure sky
set PHOTAN=10
# Width of annulus to measure sky
set PHOTDAN=10

if ( ( $#argv != 4 ) && ( $#argv != 3 ) ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name) [(coordinate file)]"
    exit 1
endif

set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=$HEADER".fits"
if ( $#argv == 4 ) then
 set TARGET=$4
else
 set TARGET="AUTO"
endif

set PHOTTABLE=RESULTS/phot.$HEADER
set OFFSETTABLE=RESULTS/offset.$HEADER

set MLCOMBTABLE=mlcomb.tbl

set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

set DITH=20
@ DITH2 = $DITH * 2

set TMPFILELIST=tmpfile.list
if( -f $TMPFILELIST ) then
  \rm $TMPFILELIST
endif

set CLLOG=log/cl`date +%y%m%d%H%M%S`.log
set CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if( !(-e log) ) then
  mkdir log
endif
if( -f $CLLOG ) then
  rm $CLLOG
endif

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

#datapars
echo scale,r,h,1.,0. > tmpdata.par
echo fwhmpsf,r,h,10,0. >> tmpdata.par
echo emission,b,h,yes >> tmpdata.par
echo sigma,r,h,INDEF >> tmpdata.par
echo datamin,r,h,INDEF >> tmpdata.par
echo datamax,r,h,INDEF >> tmpdata.par
echo noise,s,h,\"poisson\" >> tmpdata.par
echo ccdread,s,h,\"\" >> tmpdata.par
echo gain,s,h,\"GAIN\" >> tmpdata.par
echo readnoise,r,h,15 >> tmpdata.par
echo epadu,r,h,1 >> tmpdata.par
echo exposure,s,h,\"EXPTIME\" >> tmpdata.par
echo airmass,s,h,\"AIRMASS\" >> tmpdata.par
echo filter,s,h,\"FILTER01\" >> tmpdata.par
echo obstime,s,h,\"\" >> tmpdata.par
echo itime,r,h,1. >> tmpdata.par
echo xairmass,r,h,INDEF >> tmpdata.par
echo ifilter,s,h,INDEF >> tmpdata.par
echo otime,s,h,INDEF >> tmpdata.par

#centerpars
echo calgorithm,s,h,\"centroid\" > tmpcenter.par
echo cbox,r,h,5 >> tmpcenter.par
echo cthreshold,r,h,0 >> tmpcenter.par
echo minsnratio,r,h,1.,0 >> tmpcenter.par
echo cmaxiter,i,h,10 >> tmpcenter.par
echo maxshift,r,h,1 >> tmpcenter.par
echo clean,b,h,no >> tmpcenter.par
echo rclean,r,h,1 >> tmpcenter.par
echo rclip,r,h,2. >> tmpcenter.par
echo kclean,r,h,3 >> tmpcenter.par
echo mkcenter,b,h,no >> tmpcenter.par

#fitskypars
echo salgorithm,s,h,\"mode\" > tmpfitsky.par
echo annulus,r,h,RADAN >> tmpfitsky.par
echo dannulus,r,h,$PHOTDAN >> tmpfitsky.par
echo skyvalue,r,h,0. >> tmpfitsky.par
echo smaxiter,i,h,10 >> tmpfitsky.par
echo sloclip,r,h,0. >> tmpfitsky.par
echo shiclip,r,h,0. >> tmpfitsky.par
echo snreject,i,h,50 >> tmpfitsky.par
echo sloreject,r,h,3. >> tmpfitsky.par
echo shireject,r,h,3. >> tmpfitsky.par
echo khist,r,h,3. >> tmpfitsky.par
echo binsize,r,h,0.1 >> tmpfitsky.par
echo smooth,b,h,no >> tmpfitsky.par
echo rgrow,r,h,0. >> tmpfitsky.par
echo mksky,b,h,no >> tmpfitsky.par

#photpars
echo weighting,s,h,\"constant\" > tmpphot.par
echo apertures,s,h,RADAP >> tmpphot.par
echo zmag,r,h,25. >> tmpphot.par
echo mkapert,b,h,no >> tmpphot.par

echo tmpphot.par >> $TMPFILELIST
echo tmpdata.par >> $TMPFILELIST
echo tmpfitsky.par >> $TMPFILELIST
echo tmpcenter.par >> $TMPFILELIST
echo tmpphot2.par >> $TMPFILELIST
echo tmpfitsky2.par >> $TMPFILELIST

set TMPPHOT=tmp.phot
set TMPPHOT2=tmp2.phot
set TMPPSF=tmp.psf
set TMPPSF2=tmp2.psf
set COORDBASE=base.coo
echo $TMPPSF >> $TMPFILELIST
echo $TMPPSF2 >> $TMPFILELIST
echo $COORDBASE >> $TMPFILELIST


echo "### Photometry ###"
echo "### Photometry ###" >> $CLLOG

if ( $TARGET == "AUTO") then
  echo "# Object Detection #"
  echo "# Object Detection #" >> $CLLOG
  #calculate background sigma
  echo "del tmp.sigma" > $CLCOM
  echo "imstatrej RESULTS/"$OUTIM"[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG  

  set SIGMA=`gawk '{if(NR==1)print($3*4)}' tmp.sigma`

  echo "noao" > $CLCOM
  echo "digiphot" >> $CLCOM
  echo "apphot" >> $CLCOM
  echo "del tmp.tot.coo" >> $CLCOM
  echo "imdel tmp1.fits,tmp1.imh" >> $CLCOM
  echo "gauss RESULTS/$OUTIM tmp1 2.5" >> $CLCOM
  if ($DITH > 0) then
    echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant">> $CLCOM
    echo "imshift tmp1 tmp1 -"$DITH2" 0 boundary=constant" >> $CLCOM
    echo "imshift tmp1 tmp1 0 -"$DITH2" boundary=constant" >> $CLCOM
    echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant" >> $CLCOM
  endif
  echo "displ RESULTS/$OUTIM 1 fi+" >> $CLCOM
  echo "daofind tmp1 output="tmp.tot.coo" fwhmpsf=15 thresho=4 verify- sigma="$SIGMA "sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> $CLCOM
  echo "tvmark 1 tmp.tot.coo number+" >> $CLCOM
  echo "logout" >>& $CLLOG
  cat $CLCOM | cl >>& $CLLOG  
endif

gawk '{x[NR]=$1;y[NR]=$2;if(NR==1){xmin=$1;ymin=$2;}else{ if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}}END{for(i=1;i<=NR;i++) printf("%d %d\n",x[i]-xmin,y[i]-ymin)}'  $OFFSETTABLE > tmp.offset
set XCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(xmax/2+512)}' tmp.offset`
set YCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(ymax/2+512)}' tmp.offset`
echo "# XCENTER=$XCENTER YCENTER=$YCENTER"

if ( $TARGET == "AUTO") then 
  grep -v "^#" tmp.tot.coo | gawk -v xc=$XCENTER -v yc=$YCENTER -v rlim=$PHOTRADIUS 'BEGIN{i=1}{x=$1-xc;y=$2-yc;r=sqrt(x*x+y*y);if(r<=rlim){print($1,$2,$3,$4,$5,$6,i);i++}}' > $COORDBASE
else
  gawk '{print($1,$2,"-1 1 -1 -1 "NR)} ' $TARGET > $COORDBASE
endif

set NLINE=`gawk 'END{print(NR)}' $COORDBASE `

if( ( $NLINE  == 0 ) && ( $TARGET == "AUTO" ) ) then
  echo " \!\! Error \!\!"
  echo "    No Star found in circle(r=$PHOTRADIUS) centered at ($XCENTER $YCENTER)"
  exit 1
endif

if( ( $NLINE == 0 ) && ( $TARGET != "AUTO" ) ) then
  echo " \!\! Error \!\!"
  echo "    No point specified in $TARGET  "
  exit 1
endif

echo "display RESULTS/"$HEADER".fits 1 fi+" > $CLCOM
echo "tvmark 1 $COORDBASE number+" >> $CLCOM
echo "logout" >>& $CLLOG 
cat $CLCOM | cl >>& $CLLOG   
echo "# Measure following stars :"
cat $COORDBASE

echo "# Aperture Photometry in Process ... #"
echo "# Aperture Photometry in Process ... #" >> $CLLOG

if ( -f $PHOTTABLE ) then
 \rm $PHOTTABLE
endif
echo "#id itime filter epadu, x y fwhm peak, stddev nsky aperture area flux, stddev nsky aperture area flux ...." >> $PHOTTABLE
set i=1
while( $i <= $NPOINT)
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' mldata.tbl > mldata.tmp.tbl
  set XOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($1)}'` 
  set YOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($2)}'` 
  set COORDFILE=$HEADER.`gawk -v i=$i 'BEGIN{printf("%3d\n",i)}'`.tbl
  gawk -v xo=$XOFFSET -v yo=$YOFFSET '{print($1-xo,$2-yo,$3,$4,$5,$6,$7)}' $COORDBASE > $COORDFILE
  echo $COORDFILE >> $TMPFILELIST
  foreach filename (`cat mldata.tmp.tbl`)
    echo " # Measureing $filename #"
    echo " # Measureing $filename #" >> $CLLOG
    set j=1
    set NAPERTURE=4
    while($j <= $NAPERTURE)
      @ APERTURE = ( $j + 1 ) * 5
      @ ANNULUS = $APERTURE + $PHOTAN
      sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
      sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
      if ( -f $TMPPHOT.$j ) then
        \rm $TMPPHOT.$j
      endif
      cat <<EOF > $CLCOM 
      noao
      digiphot
      apphot
EOF
      if( $j == 1 ) then
        cat <<EOF >> $CLCOM 
	display $filename 1
	tvmark 1 $COORDFILE number+ radii=15 mark=circle
EOF
      endif
      cat <<EOF >> $CLCOM 
      phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
      logout
EOF
      cat $CLCOM | cl >>& $CLLOG
      sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
      @ j += 1
    end
    sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com

    echo "noao" > $CLCOM 
    echo "digiphot" >> $CLCOM 
    echo "apphot" >> $CLCOM
    if( -f $TMPPSF ) then
      \rm $TMPPSF
    endif
    echo "fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify-" >> $CLCOM
    echo "logout" >> $CLCOM
    cat $CLCOM | cl >>& $CLLOG
    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2

    set j=1
    set NAPERTURE=4
    set FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
    while($j <= $NAPERTURE)
      set FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
      @ j += 1
    end
    echo "# $filename " >> $PHOTTABLE
    paste $FILELIST >> $PHOTTABLE
  end
  @ i += 1
end


set COORDFILE=$COORDBASE
set filename=RESULTS/$OUTIM
echo " # Measureing $filename #"
echo " # Measureing $filename #" >> $CLLOG
set j=1
set NAPERTURE=4
while($j <= $NAPERTURE)
  @ APERTURE = ( $j + 1 ) * 5
  @ ANNULUS = $APERTURE + $PHOTAN
  sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
  sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
  if ( -f $TMPPHOT.$j ) then
    \rm $TMPPHOT.$j
  endif
  echo "noao" > $CLCOM 
  echo "digiphot" >> $CLCOM 
  echo "apphot" >> $CLCOM
  echo "phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par" >> $CLCOM
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG
  sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
  @ j += 1
end

sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
echo "noao" > $CLCOM 
echo "digiphot" >> $CLCOM 
echo "apphot" >> $CLCOM
if( -f $TMPPSF ) then
  \rm $TMPPSF
endif
echo "fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify-" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2

set j=1
set NAPERTURE=4
set FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
while($j <= $NAPERTURE)
  set FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
  @ j += 1
end
echo "# $filename " >> $PHOTTABLE
paste $FILELIST >> $PHOTTABLE

set j=1
while($j <= $NAPERTURE)
  echo $TMPPHOT.$j >> $TMPFILELIST
  echo $TMPPHOT2.$j >> $TMPFILELIST
  @ j += 1
end

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "IRAF log written in $CLLOG"
echo "Results of photometry are saved in $PHOTTABLE"
if ( $DEBUG == 1 ) then
  foreach filename (`cat $TMPFILELIST`)
#    if ( -f $filename ) then
      \rm $filename
#    endif
  end
endif
