#!/bin/csh -f

## Set DISLAY Parameter
# AUTOMKDISP=1; Display all images
#            0; Display minimum images
if( `env | grep AUTOMKDISP |gawk 'END{print(NR)}'` == 0 ) then
set AUTOMKDISP=0
endif

## Set Debug Parameter 
# AUTOMKDEBUG=1; erase all temporary files 
#             0; leave all temporary files 
if( `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` == 0 ) then
set AUTOMKDEBUG=0
endif
 
## Parameters for Photometry
# Radius within which detect object to do photometry
set PHOTRADIUS=200
# Radius of annulus to measure sky
set PHOTAN=10
# Width of annulus to measure sky
set PHOTDAN=10

if ( $#argv != 4 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name) (ID)"
    exit 1
endif

set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=$HEADER".fits"
set ID=$4

set PHOTTABLE=RESULTS/phot.$HEADER
set OFFSETTABLE=RESULTS/offset.$HEADER

set MLCOMBTABLE=mlcomb.tbl

set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

set MASKTABLE=RESULTS/mask.$HEADER

set DITH=20
@ DITH2 = $DITH * 2

set TMPFILELIST=tmpfile.list
if( -f $TMPFILELIST ) then
  \rm $TMPFILELIST
endif

set CLLOG=log/psfmon.$HEADER.`date +%y%m%d%H%M%S`.log
set TMPLOG=tmp.log
set CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if( !(-e log) ) then
  mkdir log
endif
if( -f $CLLOG ) then
  rm $CLLOG
endif

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

# Check Architecture
set ARCH=`uname`
uname -a >> $CLLOG

# Check IRAF Version
echo "logout" | cl > $TMPLOG
set IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if( $ARCH == "Linux" ) then
    set SECSTART=`date +%s` 
else 
    set SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
endif

#datapars
echo scale,r,h,1.,0. > tmpdata.par
echo fwhmpsf,r,h,10,0. >> tmpdata.par
echo emission,b,h,yes >> tmpdata.par
echo sigma,r,h,INDEF >> tmpdata.par
echo datamin,r,h,INDEF >> tmpdata.par
echo datamax,r,h,INDEF >> tmpdata.par
echo noise,s,h,\"poisson\" >> tmpdata.par
echo ccdread,s,h,\"\" >> tmpdata.par
echo gain,s,h,\"GAIN\" >> tmpdata.par
echo readnoise,r,h,15 >> tmpdata.par
echo epadu,r,h,1 >> tmpdata.par
echo exposure,s,h,\"EXPTIME\" >> tmpdata.par
echo airmass,s,h,\"AIRMASS\" >> tmpdata.par
echo filter,s,h,\"FILTER01\" >> tmpdata.par
echo obstime,s,h,\"\" >> tmpdata.par
echo itime,r,h,1. >> tmpdata.par
echo xairmass,r,h,INDEF >> tmpdata.par
echo ifilter,s,h,INDEF >> tmpdata.par
echo otime,s,h,INDEF >> tmpdata.par

#centerpars
echo calgorithm,s,h,\"centroid\" > tmpcenter.par
echo cbox,r,h,5 >> tmpcenter.par
echo cthreshold,r,h,0 >> tmpcenter.par
echo minsnratio,r,h,1.,0 >> tmpcenter.par
echo cmaxiter,i,h,10 >> tmpcenter.par
echo maxshift,r,h,1 >> tmpcenter.par
echo clean,b,h,no >> tmpcenter.par
echo rclean,r,h,1 >> tmpcenter.par
echo rclip,r,h,2. >> tmpcenter.par
echo kclean,r,h,3 >> tmpcenter.par
echo mkcenter,b,h,no >> tmpcenter.par

#fitskypars
echo salgorithm,s,h,\"mode\" > tmpfitsky.par
echo annulus,r,h,RADAN >> tmpfitsky.par
echo dannulus,r,h,$PHOTDAN >> tmpfitsky.par
echo skyvalue,r,h,0. >> tmpfitsky.par
echo smaxiter,i,h,10 >> tmpfitsky.par
echo sloclip,r,h,0. >> tmpfitsky.par
echo shiclip,r,h,0. >> tmpfitsky.par
echo snreject,i,h,50 >> tmpfitsky.par
echo sloreject,r,h,3. >> tmpfitsky.par
echo shireject,r,h,3. >> tmpfitsky.par
echo khist,r,h,3. >> tmpfitsky.par
echo binsize,r,h,0.1 >> tmpfitsky.par
echo smooth,b,h,no >> tmpfitsky.par
echo rgrow,r,h,0. >> tmpfitsky.par
echo mksky,b,h,no >> tmpfitsky.par

#photpars
echo weighting,s,h,\"constant\" > tmpphot.par
echo apertures,s,h,RADAP >> tmpphot.par
echo zmag,r,h,25. >> tmpphot.par
echo mkapert,b,h,no >> tmpphot.par

echo tmpphot.par >> $TMPFILELIST
echo tmpdata.par >> $TMPFILELIST
echo tmpfitsky.par >> $TMPFILELIST
echo tmpcenter.par >> $TMPFILELIST
echo tmpphot2.par >> $TMPFILELIST
echo tmpfitsky2.par >> $TMPFILELIST

set TMPPHOT=tmp.phot
set TMPPHOT2=tmp2.phot
set TMPPSF=tmp.psf
set TMPPSF2=tmp2.psf
set TMPCUR=tmp.cur
set COORDBASE=base.coo
echo $TMPPSF >> $TMPFILELIST
echo $TMPPSF2 >> $TMPFILELIST
echo $COORDBASE >> $TMPFILELIST


echo "### Photometry ###"
echo "### Photometry ###" >> $CLLOG

gawk '{x[NR]=$1;y[NR]=$2;if(NR==1){xmin=$1;ymin=$2;}else{ if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}}END{for(i=1;i<=NR;i++) printf("%d %d\n",x[i]-xmin,y[i]-ymin)}'  $OFFSETTABLE > tmp.offset
set XCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(xmax/2+512)}' tmp.offset`
set YCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(ymax/2+512)}' tmp.offset`
echo "# XCENTER=$XCENTER YCENTER=$YCENTER"

### modify koko
gawk -v id=$ID '{if($1==id)print($2,$3,"-1 1 -1 -1",$1)} ' $MASKTABLE > $COORDBASE

set NLINE=`gawk 'END{print(NR)}' $COORDBASE `

set ERRSTAT=0

if ( $AUTOMKDISP == 1 ) then
    cat << EOF > $CLCOM
	display RESULTS/${HEADER}.fits 1 fi+
	tvmark 1 $COORDBASE number+
	logout
EOF
    cat $CLCOM | cl >& $TMPLOG
    cat $TMPLOG >> $CLLOG
    set ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
endif

if ( $ERRSTAT != 0 && $AUTOMKDISP == 1 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
endif

echo "# Measure following stars :"
cat $COORDBASE

echo "# Aperture Photometry in Process ... #"
echo "# Aperture Photometry in Process ... #" >> $CLLOG

if ( -f $PHOTTABLE ) then
 \rm $PHOTTABLE
endif
echo "#id itime filter epadu, x y fwhm peak, stddev nsky aperture area flux, stddev nsky aperture area flux ...." >> $PHOTTABLE
set i=1
while( $i <= $NPOINT)
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' mldata.tbl > mldata.tmp.tbl
  set XOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($1)}'` 
  set YOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($2)}'` 
  set COORDFILE=$HEADER.`gawk -v i=$i 'BEGIN{printf("%3d\n",i)}'`.tbl
  gawk -v xo=$XOFFSET -v yo=$YOFFSET '{print($1-xo,$2-yo,$3,$4,$5,$6,$7)}' $COORDBASE > $COORDFILE
  echo $COORDFILE >> $TMPFILELIST
  foreach filename (`cat mldata.tmp.tbl`)
    echo " # Measureing $filename #"
    echo " # Measureing $filename #" >> $CLLOG
    set j=1
    set NAPERTURE=4
    while($j <= $NAPERTURE)
      @ APERTURE = ( $j + 1 ) * 5
      @ ANNULUS = $APERTURE + $PHOTAN
      sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
      sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
      if ( -f $TMPPHOT.$j ) then
        \rm $TMPPHOT.$j
      endif
      cat <<EOF > $CLCOM 
      noao
      digiphot
      apphot
EOF
      if( $j == 1 && $AUTOMKDISP == 1 ) then
        cat <<EOF >> $CLCOM 
	display $filename 1
	tvmark 1 $COORDFILE number+ radii=15 mark=circle
EOF
      endif
      if( $j == 1 ) then
	gawk 'BEGIN{printf("# Coords:")}{printf("\t%s\t%s\n",$1,$2)}' $COORDFILE 
      endif
      cat <<EOF >> $CLCOM 
      phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
      logout
EOF
      cat $CLCOM | cl >& $TMPLOG
      cat $TMPLOG >> $CLLOG
      set ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
    
      if ( $ERRSTAT != 0 ) then
	echo "# Error in IRAF ... Exiting #"
	echo "# Check $CLLOG. #" 
	exit 1
      endif

      sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
      @ j += 1
    end
    sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
    sed '/^#/d' $TMPPHOT.1 | gawk '{j=NR%5;if(j==2){printf(": %s %s a\n: center yes\n: fittype moffat\n: iterations 3\n",$1,$2)}}' > $TMPCUR

    if( -f $TMPPSF ) then
      \rm $TMPPSF
    endif

    cat <<EOF > $CLCOM
#    echo "noao" > $CLCOM 
#    echo "digiphot" >> $CLCOM 
#    echo "apphot" >> $CLCOM
#   fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify-
    imexam $filename logfile=$TMPPSF keeplog+ graphcur- wcs="physical" graphic="stdgraph" imagecur=$TMPCUR use_dis-
    logout
EOF

    cat $CLCOM | cl >& $TMPLOG
    cat $TMPLOG >> $CLLOG
    set ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

    if ( $ERRSTAT != 0 ) then
	echo "# Warning... Failed to fit PSF in $filename #"
	exit 1
    endif

#    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2
# print x,y,FWHM(moffat),peak
    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{printf("%s %s %s %s\n",$1,$2,$14,$9);i++}' > $TMPPSF2

    set j=1
    set NAPERTURE=4
    set FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
    while($j <= $NAPERTURE)
      set FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
      @ j += 1
    end
    echo "# $filename " >> $PHOTTABLE
    paste $FILELIST >> $PHOTTABLE
  end
  @ i += 1
end


set COORDFILE=$COORDBASE
set filename=RESULTS/$OUTIM
echo " # Measureing $filename #"
echo " # Measureing $filename #" >> $CLLOG
set j=1
set NAPERTURE=4
while($j <= $NAPERTURE)
  @ APERTURE = ( $j + 1 ) * 5
  @ ANNULUS = $APERTURE + $PHOTAN
  sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
  sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
  if ( -f $TMPPHOT.$j ) then
    \rm $TMPPHOT.$j
  endif
  cat << EOF > $CLCOM 
    noao
    digiphot
    apphot
    phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
    logout
EOF

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  set ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if ( $ERRSTAT != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  endif

  sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
  @ j += 1
end

sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
sed '/^#/d' $TMPPHOT.1 | gawk '{j=NR%5;if(j==2){printf(": %s %s a\n: center yes\n: fittype moffat\n: iterations 3\n",$1,$2)}}' > $TMPCUR
echo 
if( -f $TMPPSF ) then
  \rm $TMPPSF
endif
cat <<EOF > $CLCOM 
#    echo "noao" > $CLCOM 
#    echo "digiphot" >> $CLCOM 
#    echo "apphot" >> $CLCOM
# fitpsf $filename 20 coords=$COORDFILE output=$TMPPSF interac- verify- nreject=3
 imexam $filename logfile=$TMPPSF keeplog+ graphcur- wcs="physical" graphic="stdgraph" imagecur=$TMPCUR use_dis-
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
set ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if ( $ERRSTAT != 0 ) then
    echo "# Warning... Failed to fit PSF in $filename #"
    exit 1
endif

#sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{j=NR%3;if(j==2){printf("%s %s %.2f %s\n",$1,$2,2.354820045*$3,$4);i++}}' > $TMPPSF2
# print x,y,FWHM(moffat),peak
    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{printf("%s %s %s %s\n",$1,$2,$14,$9);i++}' > $TMPPSF2

set j=1
set NAPERTURE=4
set FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
while($j <= $NAPERTURE)
  set FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
  @ j += 1
end
echo "# $filename " >> $PHOTTABLE
paste $FILELIST >> $PHOTTABLE

set j=1
while($j <= $NAPERTURE)
  echo $TMPPHOT.$j >> $TMPFILELIST
  echo $TMPPHOT2.$j >> $TMPFILELIST
  @ j += 1
end

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "IRAF log written in $CLLOG"
echo "Results of photometry are saved in $PHOTTABLE"
if ( $AUTOMKDEBUG == 1 ) then
  foreach filename (`cat $TMPFILELIST`)
#    if ( -f $filename ) then
      \rm $filename
#    endif
  end
endif

#END time
if( $ARCH == "Linux" ) then
    set SECEND=`date +%s` 
else
    set SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
endif

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}' >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'

